///File : MainPanel.hpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#if not defined MAIN_PANEL_HPP_DEVIS_MAKER
#define MAIN_PANEL_HPP_DEVIS_MAKER
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
//------------------------------------------------------------------------------
#include <wx/vscroll.h>
#include <wx/event.h>
#include <wx/stdpaths.h>
#include <wx/statline.h>
#include <wx/listctrl.h>
#include <wx/wupdlock.h>
//------------------------------------------------------------------------------
#include <DevisManager.hpp>
//------------------------------------------------------------------------------
#include "strWxStdConv.hpp"
//------------------------------------------------------------------------------
#include "config.hpp"
#include "buildVersionOptions.hpp"
#include "findFile.hpp"
#include "ScrollPanelItems.hpp"
#include "MainFrame.hpp"
//------------------------------------------------------------------------------

class MainPanel;
class MainFrame;

class MainPanel : public wxPanel
{
public:
    #if defined USE_MULTIPLE_FILE
    MainPanel(wxWindow* parent, MainFrame* mainFrame, DevisManager *devisManager, const int &devisId, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTAB_TRAVERSAL );
    #else
    MainPanel(MainFrame* parent, DevisManager *devisManager, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTAB_TRAVERSAL );
    #endif

    void OnAddSection(wxCommandEvent& event);
    void OnRemoveSection(wxCommandEvent& event);
    void OnSectionUp(wxCommandEvent& event);
    void OnSectionDown(wxCommandEvent& event);
    void OnSectionRename(wxEvent& event);
    void OnListClick(wxCommandEvent& event);
    void OnListKeyDown(wxKeyEvent& event);

    void MyRefresh();
    void RefreshLabels();
    void RefreshEnable();
    void RefreshList();

    #if defined USE_MULTIPLE_FILE
    int GetDevisId();
    #endif

protected:
    #if defined USE_MULTIPLE_FILE
    int m_devisId;
    #endif

    DevisManager *m_DevisManager;

    wxBoxSizer* m_sizerInfo;

    wxStaticText *m_infoNbSection, *m_infoPrice;
    wxButton *m_btAddSection, *m_btRemoveSection, *m_btSectionUp, *m_btSectionDown, *m_btSectionRename;
    wxListBox *m_SectionsList;

    ScrollPanelItems *m_ScrollPanelItems;

    MainFrame *m_MainFrame; // Evite les problèmes de parents en cas de multi fichier...
};

#endif //MAIN_PANEL_HPP_DEVIS_MAKER
