///File : Printing.hpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#if not defined PRINTING_HPP_DEVIS_MAKER
#define PRINTING_HPP_DEVIS_MAKER

//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
//------------------------------------------------------------------------------
#include <wx/print.h>
#include <wx/stdpaths.h>

#if !wxUSE_PRINTING_ARCHITECTURE
    #error "You must set wxUSE_PRINTING_ARCHITECTURE to 1 in setup.h, and recompile the library."
#endif

#include <ctype.h>
#include "wx/metafile.h"
#include "wx/printdlg.h"
#include "wx/image.h"
#include "wx/accel.h"

#ifdef __WXMAC__
#include "wx/mac/printdlg.h"
#endif

#if wxUSE_LIBGNOMEPRINT
#include "wx/html/forcelnk.h"
FORCE_LINK(gnome_print)
#endif
//------------------------------------------------------------------------------
#include <DevisManager.hpp>
//------------------------------------------------------------------------------
#include <algorithm>
//------------------------------------------------------------------------------
#include "strWxStdConv.hpp"
//------------------------------------------------------------------------------
#include "mdo.hpp"
#include "MainFrame.hpp"
#include "findFile.hpp"
//------------------------------------------------------------------------------
#include "config.hpp"
#include "buildVersionOptions.hpp"
//------------------------------------------------------------------------------

/*#if defined __WXGTK__
    /*#define MARGING_TOP     150
    #define MARGING_BOTTOM  124
    #define MARGING_LEFT    148
    #define MARGING_RIGHT   106*/
    /*#define MARGING_TOP     150
    #define MARGING_BOTTOM  124
    #define MARGING_LEFT    130
    #define MARGING_RIGHT   130
#else
    #define MARGING_TOP     0
    #define MARGING_BOTTOM  0
    #define MARGING_LEFT    0
    #define MARGING_RIGHT   5
#endif*/

#define ITEM_NAME_COL_POS 1

struct myPos
{
    int x;
    int y;
    int page;
};

struct StructElementsPos
{
	// Const
	wxSize pageSize;
	wxRect pageWriteableRect;

	// Obj
	myPos preInfoPos;
	myPos firstItemTblPos;
	myPos tvaRecapTblPos;
	myPos postInfoPos;
};

struct StructFonts
{
    StructWxColorFont tableItem;
    StructWxColorFont tableTitle;
    StructWxColorFont tableSectionTitle;
};

struct StructTblCase
{
    wxString value;
    StructWxColorFont *font;
    int vAlign; // Left / Right / Center
    int hAlign; // Top / Bottom / Center
    int extraVMarging;
};

struct StructTblLine
{
    bool isHline;
    int height;
    int interLineSpace;
    std::vector<StructTblCase> cases;
};

struct StructTable
{
    std::vector<StructTblLine> lines;
	std::vector<int> vLinesPos;
	std::vector<int> colsWidth;
	bool doDrawVLine;
	wxRect rect;
};

class MainFrame;  //On déclare MainFrame avant pr pouvoir l'utiliser juste après.

class DevisPrintout : public wxPrintout
{
public:
    DevisPrintout();
    ~DevisPrintout();
    DevisPrintout(wxString title, DevisManager *devisManager);
    bool OnPrintPage(int page);
    bool HasPage(int page);
    bool OnBeginDocument(int startPage, int endPage);
    void GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo);

protected:
    // Draw
    void DrawPreInfo();
    void DrawPostInfo();
    #if defined USE_DEMO_TEXT_IN_PRINTOUT
    void DrawDemoText();
    void DrawDemoLogo();
    #endif

    void DrawTable(const StructTable &table);

    // Prepare
    void PrepareAll();
    void PrepareTVATable();
    void PrepareRecapTable();
    void PrepareItemsTable();
    void PrepareElementsPos();
    void PrepareFonts();
    void PreparePage();

    void PreparePreInfo();
    void PreparePostInfo();
    // Table
    void TableCalculHeightAndPos(StructTable &table);
    int TableGetColMaxWidth(const StructTable &table, const int &col);
    void TableCut(const StructTable &table, std::vector<StructTable> &tableList, const int &maxHeight, const int &maxFirstPageHeight = -1, const bool &forceMaxHeightForFirstOnes = true, const bool &preserveTitle = true);
    void TableClear(StructTable &table);


    DevisManager *m_DevisManager;

	// Content
    StructTable m_TVATbl;
	StructTable m_RecapTbl;
    std::vector<StructTable> m_ItemsTbls;

	StructElementsPos m_elPos;
	StructFonts m_fonts;

    int m_nbPage;
    /*int firstPageTableHeight;
    int lastPageTableHeight;
    bool m_tableOnLastPage;
    bool m_tableOnFirstPage;
    bool m_recapOnLastPage;*/

    int m_firstPageTableHeight;

    ConfigManager *m_dvCfg;
    ConfigManager *m_generalCfg;


    // Mdo
    mdo::Sizer *m_preInfoSizer, *m_postInfoSizer;
};

#endif //PRINTING_HPP_DEVIS_MAKER
